@@ -1131,7 +1131,7 @@ determineOutputType(struct pngx *       const pngxP,
                 *depthP = 4;
                 strcpy(tupleType, "RGB_ALPHA");
             } else {
-                *depthP = 1;
+                *depthP = 2;
                 strcpy(tupleType, "GRAYSCALE_ALPHA");
             }
         } else {
@@ -1248,7 +1248,7 @@ paletteAlpha(struct pngx * const pngxP,
 
 
 
-#define GET_PNG_VAL(p) getPngVal(&(p), pngx_colorType(pngxP))
+#define GET_PNG_VAL(p) getPngVal(&(p), pngx_bitDepth(pngxP))
 
 
 
