@@ -10,7 +10,7 @@
        does it in other libc's).  pm_config.h defines TMPDIR as P_tmpdir
        in some environments.
     */
-#define _XOPEN_SOURCE 500    /* Make sure ftello, fseeko are defined */
+#define _XOPEN_SOURCE 600    /* Make sure ftello, fseeko are defined */
 #define _LARGEFILE_SOURCE 1  /* Make sure ftello, fseeko are defined */
 #define _LARGEFILE64_SOURCE 1 
 #define _FILE_OFFSET_BITS 64
@@ -112,6 +112,30 @@ tmpDir(void) {
 
 
 
+static int
+tempFileOpenFlags(void) {
+/*----------------------------------------------------------------------------
+  Open flags (argument to open()) suitable for a new temporary file  
+-----------------------------------------------------------------------------*/
+    int retval;
+
+    retval = 0
+        | O_CREAT
+        | O_RDWR
+#if !MSVCRT
+        | O_EXCL
+#endif
+#if MSVCRT
+        | O_BINARY
+#endif
+        ;
+
+    return retval;
+}
+
+
+
+
 static int
 mkstempx(char * const filenameBuffer) {
 /*----------------------------------------------------------------------------
@@ -146,9 +170,10 @@ mkstempx(char * const filenameBuffer) {
         else {
             int rc;
 
-            rc = open(filenameBuffer, O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
+            rc = open(filenameBuffer, tempFileOpenFlags(),
+                      PM_S_IWUSR | PM_S_IRUSR);
 
-            if (rc == 0) {
+            if (rc >= 0) {
                 fd = rc;
                 gotFile = TRUE;
             } else {
@@ -240,7 +265,7 @@ pm_make_tmpfile_fd(int *         const fdP,
     pm_asprintf(&filenameTemplate, "%s%s%s%s", 
                 tmpdir, dirseparator, pm_progname, "_XXXXXX");
 
-    if (filenameTemplate == NULL)
+    if (filenameTemplate == pm_strsol)
         pm_asprintf(&error,
                     "Unable to allocate storage for temporary file name");
     else {
